--------------------------------------------------------------------------
-- Scheduler.lua 
--------------------------------------------------------------------------
--[[
Event Scheduling
]]--

function MxFH.Schedule(sName, iDelayTime, bCombatOnly, functionEvent, ...)
	local name = sName;
	if not (name) then
		name = tostring(math.random());
	end
	
	local event = { 
		Name = name;
		Time = GetTime() + iDelayTime;
		Script = functionEvent;
		CombatOnly = bCombatOnly;
		Arguments = MxFH.Pack(...);
	};
		
	MxFH.ClearEvent(name);
	tinsert(MxFH.ScheduleQueue, event);
	MxFHFrame:SetScript("OnUpdate", MxFH.OnUpdate);
end

function MxFH.Timer(iDelayTime, sText)
	MxFH.ChatPrint("Timer activated for "..iDelayTime.." seconds.");
	MxFH.Schedule(nil, iDelayTime, false, MxFH.QueueComm, sText);
end

function MxFH.IsScheduled(sName)
	for index, event in pairs(MxFH.ScheduleQueue) do
		if (event.Name == sName) then
			return true;
		end
	end
end

function MxFH.ScheduledTimeLeft(sName)
	for index, event in pairs(MxFH.ScheduleQueue) do
		if (event.Name == sName) then
			local amount = event.Time - GetTime();
			if (amount < 0) then
				amount = 0;
			end
			return amount;
		end
	end
	return 0;
end

function MxFH.ClearQueue(bCombatOnly)
	if (bCombatOnly) then
		for i = #MxFH.ScheduleQueue, 1, -1 do
			if (MxFH.ScheduleQueue[i].CombatOnly == true) then
				MxFH.DebugPrint("Clearing queue");
				table.remove(MxFH.ScheduleQueue, i);
			end
		end
	else
		MxFH.ScheduleQueue = { };	
	end
end

function MxFH.ClearEvent(sName)
	for i = #MxFH.ScheduleQueue, 1, -1 do
		if (MxFH.ScheduleQueue[i].Name == sName) then
			MxFH.DebugPrint("Removing event "..tostring(sName));
			table.remove(MxFH.ScheduleQueue, i);
		end
	end
end

function MxFH.ClearEventStartsWith(sName)
	for i = #MxFH.ScheduleQueue, 1, -1 do
		if (MxFH.ScheduleQueue[i].Name == sName) then
			MxFH.DebugPrint("Removing event "..tostring(sName));
			table.remove(MxFH.ScheduleQueue, i);
		end
	end
end

function MxFH.OnUpdate()
	local currentTime = GetTime();
	for index, event in pairs(MxFH.ScheduleQueue) do
		if (event.Time <= currentTime) then
			MxFH.DebugPrint("Fire event "..tostring(event.Name));
			if (event.Script) then
				MxFH.DebugPrint("Execute event function "..tostring(event.Name));
				event.Script(unpack(event.Arguments, 1, event.n));
			end
			MxFH.ClearEvent(event.Name);
			MxFH.OnUpdate();
			return;
		end
	end

	if (#MxFH.ScheduleQueue <= 0) then
		MxFHFrame:SetScript("OnUpdate", nil);
	end	
end

function MxFH.Pack(...)
    return { n = select('#', ...), ... };
end
