--------------------------------------------------------------------------
-- SlaveGirl.lua 
--------------------------------------------------------------------------
--[[
Multibox Follower Helper
Author: Zensunim of Dragonblight

Change Log:
	v1.0 
		- Initial Release
	v1.1
		- Updated sounds to play on the Master channel instead of the SFX channel
	v1.1.1
		- Updated TOC for 4.1
	v1.1.2
		- Updated TOC for 4.2
	v1.1.3
		- Fixed bug with following a character
	v1.1.4
		- Updated TOC for 4.3
	v1.1.5
		- Updated for 5.0.4
	v1.2
		- Modified HP/Mana alerts to use less CPU
		- Updated for 5.1
	v1.2.1
		- Updated for 5.2
	v1.2.2
		- Updated for 5.3
	v1.2.3
		- Updated for 5.4.7
	v1.2.4
		- Updated for 6.0
	v1.2.5
		- Converted all sounds to .ogg format
	v1.3
		- Fixed issues with same/cross-realm communications
		- Fixed issue with auto-completing quests with a single reward
		- Fixed issue with setting a new master in the options
		- Fixed issue with sharing quests when an auto-opened quest is being displayed
		- /SG now displays the current master
	v1.3.1
		- Fixed issue with setting a new master on the same realm in a group
	v1.3.2
		- Fixed party spam issue with instances
	v1.3.3
		- Updated for 6.2
	v1.3.4
		- Updated for 7.0
	v1.4
		- Updated for 7.1
		- Fixed issue with sounds overlapping and being extremely loud
	v1.4.1
		- Fixed LUA error issue with picking up unsharable quests
	v2.0
		- Improved following system to not spam noises when following while following
	v2.0.1
		- Updated for 7.2
	v2.1
		- Added the ability to leave group when the master leaves group
		- Added the ability to suppress role selection and custom group role popups
	v2.1.1
		- Fixed a bug with assigning the master user not including the realm name
	v2.2
		- Updated for 7.3
		- Replaced UnitMana(...) with UnitPower(...,0) because UnitMana was removed
	v2.3
		- Updated for 8.0
	v2.3.1
		- Updated ToC for 8.0
	v2.3.2
		- Fixed a bug with game-forced auto-accepted quests not being auto-accepted
	v2.3.3
		- Updated ToC for 8.1
	v2.4
		- Classic WoW support
	v2.4.1
		- Updated ToC for 8.2
	v2.4.2
		- Classic WoW 1.13.2 support
	v2.5
		- Updated ToC for 8.2.5
		- Fixed a bug with spamming following while mounted
	v2.6
		- Updated ToC for 8.3
		- Fixed a bug with spamming area quests
	v2.7
		- Updated ToC for 9.0
	v2.7.1
		- Updated ToC for 9.2

Configuration Commands:
	/SG options - Display the options menu
	/SG master - Follow orders from this player

]]--

if (MxFH) then
	-- Do nothing
else
	MxFH = { };
end

SlaveGirlOptions = { };

MxFH = {
	DefaultSetting = {
		LowHealthThreshold = 50;
		LowHealthCooldown = 15;
		LowManaThreshold = 40;
		LowManaCooldown = 25;
		Monitor = true;
		Broadcast = nil;
		StartFollowSound = true;
		StopFollowSound = true;
		LowHealthSound = true;
		LowManaSound = true;
		AutoAcceptGroupInvite = true;
		AutoAcceptQuest = true;
		AutoShareQuest = true;
		AutoTurnInQuest = nil;
		AutoLeaveGroup = nil;
		AutoRole = nil;
		Master = nil;
	};

-- **********************************************************************************************
--
-- DON'T EDIT BELOW THIS LINE
--
-- **********************************************************************************************

	Version = "2.7"; -- Version number (text format)
	DataCode = "1"; -- Saved Variable versioning, change this value to force a reset to default
	ModCode = "KSG"; -- Mod communication code
	Setting = { };
	ScheduleQueue = { };

	Data = {
		LastFollowingTarget = nil;
		TimerPowerInfusion = 0;
		TimerLowHealth = GetTime();
		TimerLowMana = GetTime();
		Me = UnitName("player");
		Realm = gsub(GetRealmName(), "%s", "");
		MeFull = UnitName("player").."-"..gsub(GetRealmName(), "%s", "");
		InGroup = (GetNumGroupMembers() > 0);
		FollowerInGroup = nil;
		MasterInGroup = nil;
		Followers = { };
		LowHealthThreshold = 0;
		LowManaThreshold = 0;
		TimerSound = { };
		DebugMode = nil;
		LastFollowingAction = nil;
		AutoLeaveGroupIgnore = nil;
		BetaMode = nil;
		ClassicMode = nil;
	};
	
	Sounds = {
		["StartFollow"] = "Interface\\AddOns\\SlaveGirl\\Sounds\\follow_on.ogg",
		["StopFollow"] = "Interface\\AddOns\\SlaveGirl\\Sounds\\follow_off.ogg",
		["LowMana"] = "Interface\\AddOns\\SlaveGirl\\Sounds\\low_mana.ogg",
		["LowHealth"] = "Interface\\AddOns\\SlaveGirl\\Sounds\\low_health.ogg",
	};
}

if (select(4, GetBuildInfo()) >= 90200) then
	MxFH.Data.BetaMode = true;
end
if (select(4, GetBuildInfo()) < 20000) then
	MxFH.Data.ClassicMode = true;
end


function MxFH_ResetDefaults()
	MxFH.Setting.LowHealthThreshold = MxFH.DefaultSetting.LowHealthThreshold;
	MxFH.Setting.LowHealthCooldown = MxFH.DefaultSetting.LowHealthCooldown;
	MxFH.Setting.LowManaThreshold = MxFH.DefaultSetting.LowManaThreshold;
	MxFH.Setting.LowManaCooldown = MxFH.DefaultSetting.LowManaCooldown;
	MxFH.Setting.Monitor = MxFH.DefaultSetting.Monitor;
	MxFH.Setting.Broadcast = MxFH.DefaultSetting.Broadcast;
	MxFH.Setting.StartFollowSound = MxFH.DefaultSetting.StartFollowSound;
	MxFH.Setting.StopFollowSound = MxFH.DefaultSetting.StopFollowSound;
	MxFH.Setting.LowHealthSound = MxFH.DefaultSetting.LowHealthSound;
	MxFH.Setting.LowManaSound = MxFH.DefaultSetting.LowManaSound;
	MxFH.Setting.AutoAcceptGroupInvite = MxFH.DefaultSetting.AutoAcceptGroupInvite;
	MxFH.Setting.AutoAcceptQuest = MxFH.DefaultSetting.AutoAcceptQuest;
	MxFH.Setting.AutoShareQuest = MxFH.DefaultSetting.AutoShareQuest;
	MxFH.Setting.AutoTurnInQuest = MxFH.DefaultSetting.AutoTurnInQuest;
	MxFH.Setting.AutoLeaveGroup = MxFH.DefaultSetting.AutoLeaveGroup;
	MxFH.Setting.AutoRole = MxFH.DefaultSetting.AutoRole;
	MxFH.Setting.Master = MxFH.DefaultSetting.Master;
end

MxFH_ResetDefaults();

function MxFH_OnEvent(self, event, ...)
	if (event == "VARIABLES_LOADED") then
		C_ChatInfo.RegisterAddonMessagePrefix(MxFH.ModCode);
		MxFH.DebugPrint("MFH "..MxFH.Version.." variables loaded.");
		MxFHTooltip:SetOwner(UIParent, "ANCHOR_NONE");
		MxFHFrame:RegisterEvent("START_AUTOREPEAT_SPELL");
		MxFHFrame:RegisterEvent("STOP_AUTOREPEAT_SPELL");
		MxFHFrame:RegisterEvent("UI_ERROR_MESSAGE");
		MxFHFrame:RegisterEvent("AUTOFOLLOW_BEGIN");
		MxFHFrame:RegisterEvent("AUTOFOLLOW_END");
		MxFHFrame:RegisterEvent("PARTY_INVITE_REQUEST");
		MxFHFrame:RegisterEvent("GROUP_ROSTER_UPDATE");
		MxFHFrame:RegisterEvent("QUEST_DETAIL");
		MxFHFrame:RegisterEvent("QUEST_ACCEPTED");
		MxFHFrame:RegisterEvent("QUEST_PROGRESS");
		MxFHFrame:RegisterEvent("QUEST_COMPLETE");
		MxFHFrame:RegisterEvent("CHAT_MSG_ADDON");
		MxFHFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
		MxFHFrame:RegisterEvent("UNIT_HEALTH");
		MxFHFrame:RegisterEvent("UNIT_POWER_UPDATE");
		if not (MxFH.Data.ClassicMode) then
			MxFHFrame:RegisterEvent("LFG_ROLE_CHECK_SHOW");
			MxFHFrame:RegisterEvent("LFG_LIST_JOINED_GROUP");		
			MxFHFrame:RegisterEvent("LFG_LIST_ACTIVE_ENTRY_UPDATE");
		end

		if (SlaveGirlOptions.DataCode ~= MxFH.DataCode or not (SlaveGirlOptions.Setting)) then
			SlaveGirlOptions = { };
			SlaveGirlOptions.DataCode = MxFH.DataCode;
			SlaveGirlOptions.Setting = { };
			MxFH_ErrorPrint("New database detected. Clearing settings.");
		elseif (SlaveGirlOptions.Setting[MxFH.Data.Me]) then
			MxFH.DebugPrint("Profile found. Loading settings");
			if (SlaveGirlOptions.Setting[MxFH.Data.Me].LowHealthThreshold) then
				MxFH.Setting.LowHealthThreshold = SlaveGirlOptions.Setting[MxFH.Data.Me].LowHealthThreshold;
			end
			if (SlaveGirlOptions.Setting[MxFH.Data.Me].LowHealthCooldown) then
				MxFH.Setting.LowHealthCooldown = SlaveGirlOptions.Setting[MxFH.Data.Me].LowHealthCooldown;
			end
			if (SlaveGirlOptions.Setting[MxFH.Data.Me].LowManaThreshold) then
				MxFH.Setting.LowManaThreshold = SlaveGirlOptions.Setting[MxFH.Data.Me].LowManaThreshold;
			end
			if (SlaveGirlOptions.Setting[MxFH.Data.Me].LowManaCooldown) then
				MxFH.Setting.LowManaCooldown = SlaveGirlOptions.Setting[MxFH.Data.Me].LowManaCooldown;
			end
			MxFH.Setting.Monitor = SlaveGirlOptions.Setting[MxFH.Data.Me].Monitor;
			MxFH.Setting.Broadcast = SlaveGirlOptions.Setting[MxFH.Data.Me].Broadcast;
			MxFH.Setting.StartFollowSound = SlaveGirlOptions.Setting[MxFH.Data.Me].StartFollowSound;
			MxFH.Setting.StopFollowSound = SlaveGirlOptions.Setting[MxFH.Data.Me].StopFollowSound;
			MxFH.Setting.LowHealthSound = SlaveGirlOptions.Setting[MxFH.Data.Me].LowHealthSound;
			MxFH.Setting.LowManaSound = SlaveGirlOptions.Setting[MxFH.Data.Me].LowManaSound;
			MxFH.Setting.AutoAcceptGroupInvite = SlaveGirlOptions.Setting[MxFH.Data.Me].AutoAcceptGroupInvite;
			MxFH.Setting.AutoAcceptQuest = SlaveGirlOptions.Setting[MxFH.Data.Me].AutoAcceptQuest;
			MxFH.Setting.AutoShareQuest = SlaveGirlOptions.Setting[MxFH.Data.Me].AutoShareQuest;
			MxFH.Setting.AutoTurnInQuest = SlaveGirlOptions.Setting[MxFH.Data.Me].AutoTurnInQuest;
			MxFH.Setting.AutoLeaveGroup = SlaveGirlOptions.Setting[MxFH.Data.Me].AutoLeaveGroup;
			MxFH.Setting.AutoRole = SlaveGirlOptions.Setting[MxFH.Data.Me].AutoRole;
			MxFH.Setting.Master = SlaveGirlOptions.Setting[MxFH.Data.Me].Master;
		else
			MxFH_ErrorPrint("New player detected. Setting defaults.");
		end
		
		MxFH_CheckGroup(true);
		MxFH_RenderOptions();
		MxFH_SaveSettings();
		return;
	end
	if (event == "AUTOFOLLOW_BEGIN") then
		MxFH.DebugPrint("AUTOFOLLOW_BEGIN event");
		local follower = ...;
		if (MxFH.Data.ClassicMode or not UnitUsingVehicle("player")) then
			MxFH.Schedule("FollowSound", 0, nil, MxFH.StartFollowing, follower, MxFH.IsScheduled("FollowSound"));
		end
		LastFollowingAction = "BEGIN";
		return;
	end
	if (event == "AUTOFOLLOW_END") then
		MxFH.DebugPrint("AUTOFOLLOW_END event");
		local scheduled = MxFH.IsScheduled("FollowSound");
		if (scheduled and LastFollowingAction == "END") then
			MxFH.DebugPrint("Ignore duplicate end follow command");
			return;
		end
		if (MxFH.Data.ClassicMode or not UnitUsingVehicle("player")) then
			MxFH.Schedule("FollowSound", .2, nil, MxFH.StopFollowing, scheduled);
		end
		LastFollowingAction = "END";
		return;
	end		
	if (event == "PARTY_INVITE_REQUEST") then
		local partyMaster = ...;
		if (MxFH.Setting.AutoAcceptGroupInvite and MxFH_IsMaster(partyMaster)) then
			AcceptGroup();
		end
		return;
	end
	if (event == "GROUP_ROSTER_UPDATE") then
		if (MxFH.Setting.AutoAcceptGroupInvite and MxFH.Setting.Master) then
			StaticPopupDialogs["PARTY_INVITE"].inviteAccepted = 1;
			StaticPopup_Hide("PARTY_INVITE");
			StaticPopup_Hide("PARTY_INVITE_XREALM");
		end
		if not (MxFH.Data.InGroup) then
			MxFH_CheckGroup(true);
		end
		MxFH.Data.InGroup = (GetNumSubgroupMembers() > 0)
		if (MxFH.Data.InGroup) then
			MxFH_CheckGroup(false);
		end
		return;
	end
	if (event == "QUEST_DETAIL") then
		if (MxFH.Setting.AutoAcceptQuest and MxFH.Setting.Master) then
			if (UnitName("player") == UnitName("NPC")) then
				MxFH.DebugPrint("Auto-Accepting quest.");
				MxFH_SendComm("O/Q/"..GetTitleText());
				if (MxFH.Data.ClassicMode or not QuestGetAutoAccept()) then
					AcceptQuest();
				end
				HideUIPanel(QuestFrame);
			end
		end
		return;
	end
	if (event == "QUEST_ACCEPTED") then
		local index = ...;
		if (index and MxFH.Setting.AutoShareQuest and (MxFH.Data.FollowerInGroup or MxFH.Data.MasterInGroup)) then
			MxFH_ShareQuest(index);
		end
		if (MxFH.Setting.AutoTurnInQuest) then
			if (not MxFH.Data.ClassicMode and QuestGetAutoAccept()) then
				HideUIPanel(QuestFrame);
			end
		end
    return;
	end
	if (event == "QUEST_PROGRESS") then
		if (MxFH.Setting.AutoTurnInQuest) then
			MxFH.DebugPrint("Quest progress");
			if (IsQuestCompletable()) then
				CompleteQuest();
			end
		end
    return;
	end
	if (event == "QUEST_COMPLETE") then
		if (MxFH.Setting.AutoTurnInQuest) then
			MxFH.DebugPrint("Quest choices: "..GetNumQuestChoices());
			if (GetNumQuestChoices() < 2) then
			    if (GetNumQuestChoices() == 1) then
			    	GetQuestReward(1);
			    else
			    	GetQuestReward();
			    end
			    HideUIPanel(QuestFrame);
			end
		end
    return;
	end
	if (event == "CHAT_MSG_ADDON") then
		local prefix, message, messageType, sender = ...;
		if ( sender ~= MxFH.Data.Me ) then
			if (prefix == MxFH.ModCode and message and sender) then
				MxFH_ReceiveComm(message, sender);
			end
		end
		return;
	end
	if (event == "UNIT_HEALTH" or event == "UNIT_POWER" or event == "UNIT_POWER_UPDATE") then
		local unit, _ = ...;
		if (unit and UnitIsUnit(unit, "player")) then
			MxFH_Update();
		end
	end
	if (event == "UI_ERROR_MESSAGE") then
		local message = ...;
		if ( message == ERR_SPELL_COOLDOWN ) then
			MxFH_SendComm("E/C");
			return;
		end
		if ( message == ERR_SPELL_OUT_OF_RANGE ) then
			MxFH_SendComm("E/R");
			return;
		end
		if ( message == SPELL_FAILED_MOVING ) then
			MxFH_SendComm("E/V");
			return;
		end
		if ( message == SPELL_FAILED_LINE_OF_SIGHT ) then
			MxFH_SendComm("E/L");
			return;
		end
		if ( message == SPELL_FAILED_BAD_TARGETS ) then
			MxFH_SendComm("E/B");
			return;
		end
		if ( message == ERR_OUT_OF_MANA ) then
			MxFH_SendComm("E/M");
			return;
		end
		if ( message == SPELL_FAILED_UNIT_NOT_INFRONT ) then
			MxFH_SendComm("E/F");
			return;
		end
		if ( message == SPELL_FAILED_SILENCED ) or ( message == SPELL_FAILED_STUNNED ) then
			MxFH_SendComm("E/S");
			return;
		end
		if ( message == ERR_ATTACK_PACIFIED ) or ( message == ERR_ATTACK_CHARMED ) then
			MxFH_SendComm("E/S");
			return;
		end
		if ( message == ERR_ATTACK_CONFUSED ) or ( message == ERR_ATTACK_FLEEING ) then
			MxFH_SendComm("E/S");
			return;
		end
		if ( message == ERR_ATTACK_PREVENTED_BY_MECHANIC_S ) or ( message == "Can't attack while polymorphed." ) then
			MxFH_SendComm("E/S");
			return;
		end
	end
	if (event == "LFG_ROLE_CHECK_SHOW") then
		local role = select(5, GetSpecializationInfo(GetSpecialization()));
		if (MxFH.Setting.AutoRole) then
			local oldLeader = GetLFGRoles();
			SetLFGRoles(oldLeader, role == "TANK", role == "HEALER", role == "DAMAGER");
			CompleteLFGRoleCheck(true);
		end
		return;
	end
	if (event == "LFG_LIST_JOINED_GROUP") then
		MxFH.DebugPrint("LFG group joined...ignoring auto-leave for 3 seconds");
		MxFH.Data.AutoLeaveGroupIgnore = GetTime() + 3;
		if (MxFH.Setting.AutoRole) then
			StaticPopupSpecial_Hide(LFGListInviteDialog);
		end
		return;
	end
end

function MxFH.ChatPrint(str)
	DEFAULT_CHAT_FRAME:AddMessage("[SG] "..str, 0.25, 1.0, 0.25);
end

function MxFH_ErrorPrint(str)
	DEFAULT_CHAT_FRAME:AddMessage("[SG] "..str, 1.0, 0.5, 0.5);
end

function MxFH.DebugPrint(str)
	if (MxFH.Data.DebugMode) then
		DEFAULT_CHAT_FRAME:AddMessage("[SG] "..str, 0.75, 1.0, 0.25);
	end
end

function MxFH_CommPrint(str)
	DEFAULT_CHAT_FRAME:AddMessage(str, 0.45, 0.78, 0.96);
end

function MxFH_FindPattern(text, pattern)
	if (string.find(text, pattern, 1)) then
		return string.sub(text, string.find(text, pattern, 1));
	else
		return nil;
	end
end

function MxFH_OnLoad()
	SlashCmdList["SG"] = MxFH_Command;
	SLASH_SG1 = "/SG";
	SlashCmdList["MxFH"] = MxFH_Command;
	SLASH_MxFH1 = "/MxFH";
	MxFHFrame:RegisterEvent("CHAT_MSG_ADDON");
	MxFH.ChatPrint("MFH "..MxFH.Version.." loaded.");
	MxFHFrame:RegisterEvent("VARIABLES_LOADED");
end

function MxFH_Command(arg1)
	local Command = string.upper(arg1);
	local DescriptionOffset = string.find(arg1,"%s",1);
	local Description = nil;
	
	if (DescriptionOffset) then
		Command = string.upper(string.sub(arg1, 1, DescriptionOffset - 1));
		Description = tostring(string.sub(arg1, DescriptionOffset + 1));
	end
	
	MxFH.DebugPrint("Command executed: "..Command);
	
	if (Command == "MASTER") then
		MxFH_Command_Master(Description);
	elseif (Command == "MONITOR") then
		MxFH_Command_Monitor();
	elseif (Command == "FOLLOW") then
		MxFH_Command_Follow();
	elseif (Command == "BROADCAST") then
		MxFH_Command_Broadcast();
	elseif (Command == "STATUS") then
		MxFH_Command_Status();
	elseif (Command == "DEBUG") then
		MxFH_Command_Debug();
	elseif (Command == "OPTION" or Command == "OPTIONS") then
		InterfaceOptionsFrame_OpenToCategory("Multibox Helper")
		InterfaceOptionsFrame_OpenToCategory("Multibox Helper")
	elseif (Command == "HELP" or Command == "") then
		MxFH_Command_Help();
	else
		MxFH_Command_Help();
	end
end

function MxFH_Update()
		local currentTime = GetTime();
		if (UnitIsDeadOrGhost("player")) then
			return;
		end
		local hp = UnitHealth("player");
		if (hp > 1) then
			if (MxFH.Setting.LowHealthSound and currentTime > MxFH.Data.TimerLowHealth and (((hp * 100) / UnitHealthMax("player")) < MxFH.Setting.LowHealthThreshold)) then
				MxFH_PlaySound("LowHealth")
				MxFH.Data.TimerLowHealth = GetTime() + MxFH.Setting.LowHealthCooldown;
			end
			if (MxFH.Setting.LowManaSound and UnitPowerMax("player", 0) > 0 and currentTime > MxFH.Data.TimerLowMana and (((UnitPower("player", 0) * 100) / UnitPowerMax("player", 0)) < MxFH.Setting.LowManaThreshold) and (UnitPowerType("player") == 0)) then
				MxFH_PlaySound("LowMana");
				MxFH.Data.TimerLowMana = GetTime() + MxFH.Setting.LowManaCooldown;
			end
		end
end

function MxFH_Command_Follow()
	FollowUnit("target");
end

function MxFH_Command_Help()
	local currentMaster;
	if (MxFH.Setting.Master) then
		currentMaster = "|cFFEEEE00"..MxFH.Setting.Master.."|r";
	else
		currentMaster = "[|cFFEE0000Nobody|r]";
	end

	DEFAULT_CHAT_FRAME:AddMessage("[SG] Multibox Follower Helper "..MxFH.Version, 0.25, 1.0, 0.25);
	DEFAULT_CHAT_FRAME:AddMessage("Current Master: "..currentMaster, 0.25, 1.0, 0.75);
	DEFAULT_CHAT_FRAME:AddMessage("|cFFEEEE00/sg options|r -- Display options", 0.25, 1.0, 0.75);
	DEFAULT_CHAT_FRAME:AddMessage("|cFFEEEE00/sg master|r -- Set your current target as your master (or to nobody if you have no target)", 0.25, 1.0, 0.75);

end

function MxFH_Command_Master(arg1)
	if (arg1 and arg1 ~= "") then
		if (arg1 == "c" or arg1 == "C") then
			MxFH.Setting.Master = nil;
			MxFH.ChatPrint("You are no longer taking orders from anyone.");
		elseif (arg1 == MxFH.Data.Me or arg1 == MxFH.Data.MeFull) then
			MxFH_ErrorPrint("You cannot be your own master.");
		else
			MxFH.Setting.Master = arg1;
			MxFH.ChatPrint("You are now taking orders from "..arg1..".");
			MxFH_ReportToMaster(arg1);
		end
	else
		local sName, sRealm = UnitName("target");
		if (sName) then
			local fullName = MxFH_FullName("target", true)
			if (fullName == MxFH.Data.MeFull) then
				MxFH_ErrorPrint("You cannot be your own master.");
			elseif (not UnitIsPlayer("target")) then
				MxFH_ErrorPrint("Only players can be masters.");
			else
				MxFH.Setting.Master = fullName;
				MxFH.ChatPrint("You are now taking orders from "..sName..".");
				MxFH_ReportToMaster(fullName);
		end
		else
			MxFH.Setting.Master = nil;
			MxFH.ChatPrint("You are no longer taking orders from anyone.");
		end
	end
	MxFH_SaveSettings();
end

function MxFH_PlaySound(sSound)
	local currentTime = GetTime();
	local soundTime = MxFH.Data.TimerSound[sSound];
	if (not soundTime or currentTime > soundTime) then
		PlaySoundFile(MxFH.Sounds[sSound], "Master");
		MxFH.Data.TimerSound[sSound] = GetTime() + .5;
	end
end

function MxFH_Command_Monitor()
	if (MxFH.Setting.Monitor == true) then
		MxFH.Setting.Monitor = nil;
		MxFH.ChatPrint("Monitoring activity is off.");
	else
		MxFH.Setting.Monitor = true;
		MxFH.ChatPrint("Monitoring activity is on.");
	end
		MxFH.DebugPrint("Monitor loaded: "..tostring(MxFH.Setting.Monitor).."/"..tostring(SlaveGirlOptions.Setting[MxFH.Data.Me].Monitor));
	MxFH_SaveSettings();
		MxFH.DebugPrint("Monitor loaded: "..tostring(MxFH.Setting.Monitor).."/"..tostring(SlaveGirlOptions.Setting[MxFH.Data.Me].Monitor));
end

function MxFH_Command_Broadcast()
	if (MxFH.Setting.Broadcast == true) then
		MxFH.Setting.Broadcast = nil;
		MxFH.ChatPrint("Broadcasting activity is off.");
	else
		MxFH.Setting.Broadcast = true;
		MxFH.ChatPrint("Broadcasting activity is on.");
	end
	MxFH_SaveSettings();
end

function MxFH_SendComm(sNewMessage, bOverride)
	if (MxFH.Setting.Broadcast or bOverride) then	
		if (IsInGroup(LE_PARTY_CATEGORY_INSTANCE)) then
			C_ChatInfo.SendAddonMessage(MxFH.ModCode,sNewMessage,"INSTANCE_CHAT");
		elseif (UnitInBattleground("player")) then
			C_ChatInfo.SendAddonMessage(MxFH.ModCode,sNewMessage,"BATTLEGROUND");
		else		
			C_ChatInfo.SendAddonMessage(MxFH.ModCode,sNewMessage,"RAID");
		end
		--MxFH.DebugPrint("Sending Comm: "..tostring(sNewMessage));
	end
end

function MxFH_ReceiveComm(sMessage, sSender)
	local decodedMessage = MxFH_CommDecode(sMessage, sSender);
	MxFH.DebugPrint("Message from "..sSender..": "..sMessage);
	if (decodedMessage) then
		if (decodedMessage == "COMMAND:MASTER") then
			MxFH_ReportToMaster(sSender);
		elseif (decodedMessage == "COMMAND:FOLLOWER") then
			MxFH.Data.Followers[sSender] = true;
			MxFH.DebugPrint(sSender.." has reported in.");
			MxFH_CheckGroup(false);
		else
			if (MxFH.Setting.Monitor and MxFH.Data.Followers[sSender]) then
				MxFH_CommPrint("[SG] ["..MxFH.StripRealm(sSender).."]: "..decodedMessage);
			end
		end
	end
end

function MxFH_CommDecode(sText, sSender)
	if (sText == "") or not (sText) then
		return nil;
	end
	
	local DecodedMessage = nil;
	local Slash = string.find(sText,"/",1);
	local Slash2 = "";
	local Slash3 = "";
	local Command = "";
	local CommandType = "";
	local Data = "";
	local Data2 = "";
	
	if (Slash) then
		CommandType = string.sub(sText,1,Slash - 1);
		Slash2 = string.find(sText,"/",Slash + 1);
		if (Slash2) then
			Command = string.sub(sText,Slash + 1,Slash2 - 1);
			Slash3 = string.find(sText,"/",Slash2 + 1);
			if (Slash3) then
				Data = string.sub(sText,Slash2 + 1,Slash3 - 1);
				Data2 = string.sub(sText,Slash3 + 1);
			else
				Data = string.sub(sText,Slash2 + 1);
			end
		else
			Command = string.sub(sText,Slash + 1)
		end
	else
		CommandType = sText;
	end
	
	if (CommandType == "X") then
		if (Command == "W") then
			DecodedMessage = "Wand: (Stop)";
		elseif (Command == "H") then
			DecodedMessage = "Heal: Aborted: "..Data;
		else
			DecodedMessage = Command;
		end
	elseif (CommandType == "W") then
		if (Command == "S") then
			DecodedMessage = "Wand: "..Data;
		else
			DecodedMessage = Command;
		end
	elseif (CommandType == "H") then
		if (Command == "HT") then
			DecodedMessage = "Heal: Healthstone";
		elseif (MxFH_CommDecode_Spell(Command)) then
			DecodedMessage = "Heal: "..MxFH_CommDecode_Spell(Command);
			if (Data2 ~= "") then
				DecodedMessage = DecodedMessage.." ("..Data.."): "..Data2;
			elseif (Data ~= "") then
				DecodedMessage = DecodedMessage.." ("..Data..")";
			end
		else
			DecodedMessage = Command;
		end
	elseif (CommandType == "A") then
		if (MxFH_CommDecode_Spell(Command)) then
			DecodedMessage = "Attack: "..MxFH_CommDecode_Spell(Command);
			if (Data2 ~= "") then
				DecodedMessage = DecodedMessage.." ("..Data.."): "..Data2;
			elseif (Data ~= "") then
				DecodedMessage = DecodedMessage.." ("..Data..")";		
			end
		else
			DecodedMessage = Command;
		end
	elseif (CommandType == "B") then
		if (MxFH_CommDecode_Spell(Command)) then
			DecodedMessage = "Buff: "..MxFH_CommDecode_Spell(Command);
			if (Data2 ~= "") then
				DecodedMessage = DecodedMessage.." ("..Data.."): "..Data2;
			elseif (Data ~= "") then
				DecodedMessage = DecodedMessage.." ("..Data..")";		
			end
		else
			DecodedMessage = Command;
		end
	elseif (CommandType == "R") then
		DecodedMessage = "Resurrecting: "..Command;
	elseif (CommandType == "T") then
		DecodedMessage = "Target: "..Command;
		if (Command == "X") then
			DecodedMessage = "Target: "..Data.." [Unknown]";
		elseif (Command == "F") then
			DecodedMessage = "Target: "..Data.." [Friendly]";
		elseif (Command == "E") then
			DecodedMessage = "Target: "..Data.." [Enemy]";
		elseif (Command == "P") then
			DecodedMessage = "Target: "..Data.." [Party]";
		elseif (Command == "R") then
			DecodedMessage = "Target: "..Data.." [Raid]";
		elseif (Command == "S") then
			DecodedMessage = "Target: [Self]";
		elseif (Command == "M") then
			DecodedMessage = "Target: "..Data.." [Master]";
		else
			DecodedMessage = Command;
		end
		DecodedMessage = nil;
	elseif (CommandType == "F") then
		if (Command == "S") then
			if (Data) then
				DecodedMessage = "Follow: "..Data;
			else
				DecodedMessage = "Follow: [Start]";
			end
		elseif (Command == "X") then
			DecodedMessage = "Follow: [Stop]";
		else
			DecodedMessage = Command;
		end
	elseif (CommandType == "O") then
		if (Command == "T") then
			DecodedMessage = "Order: Accepting Trade";
		elseif (Command == "Q") then
			DecodedMessage = "Order: Accepting Quest ("..Data..")";
		elseif (Command == "S" and Data) then
			MxFH.DebugPrint("Requested share of quest: "..Data);
			if (MxFH.Setting.AutoAcceptQuest and MxFH.Setting.Master) then
				local title = GetTitleText();
				MxFH.DebugPrint("Got new title: "..tostring(title));
				if (title == Data) then
					MxFH.DebugPrint("Auto-Accepting quest.");
					--MxFH_SendComm("O/Q/"..GetTitleText());
					AcceptQuest();
					HideUIPanel(QuestFrame);
				end
			end	
		else
			DecodedMessage = Command;
		end
	elseif (CommandType == "S") then
		if (MxFH_CommDecode_Spell(Command)) then
			DecodedMessage = MxFH_CommDecode_Spell(Command);
			if (Data2 ~= "") then
				DecodedMessage = DecodedMessage.." ("..Data.."): "..Data2;
			elseif (Data ~= "") then
				DecodedMessage = DecodedMessage.." ("..Data..")";		
			end
		else
			DecodedMessage = Command;
		end
	elseif (CommandType == "E") then
		if (Command == "V") then
			DecodedMessage = "Error: Can't cast -- Moving";
		elseif (Command == "C") then
			DecodedMessage = "Error: Can't cast -- Cooldown";
		elseif (Command == "L") then
			DecodedMessage = "Error: Can't cast -- No line of sight";
		elseif (Command == "B") then
			DecodedMessage = "Error: Can't cast -- Bad target";
		elseif (Command == "F") then
			DecodedMessage = "Error: Can't cast -- Target isn't in front";
		elseif (Command == "I") then
			DecodedMessage = "Error: Can't attack -- Invalid/Ignored target";
		elseif (Command == "N") then
			DecodedMessage = "Error: Can't attack -- Out of range/No Script";
		elseif (Command == "R") then
			DecodedMessage = "Error: Can't cast -- Out of range";
		elseif (Command == "X") then
			DecodedMessage = "Error: Can't cast -- In combat";
		elseif (Command == "M") then
			DecodedMessage = "Error: Can't cast -- Out of mana";
		elseif (Command == "S") then
			DecodedMessage = "Error: Can't cast -- Silenced/Stunned";
		else
			DecodedMessage = Command;
		end
	elseif (CommandType == "G") then
		if (Command == "M") then
			DecodedMessage = "COMMAND:MASTER";
		end
		if (Command == "S") then
			DecodedMessage = "COMMAND:FOLLOWER";
		end
	else
		DecodedMessage = sText;
	end
	
	return DecodedMessage;
end

function MxFH_Command_Status()
	MxFH.ChatPrint("Multibox Follower Helper Status Report:");
	if (MxFH.Setting.Master) then
		MxFH.ChatPrint("- Master: "..MxFH.Setting.Master);
	else
		MxFH.ChatPrint("- Master: [None]");
	end
	MxFH.ChatPrint("- Comm Broadcasting: "..MxFH_Logic(MxFH.Setting.Broadcast));
end

function MxFH_Logic(bValue)
	if (bValue) then
		if (bValue == false) then
			return "Off";
		else
			return "On";
		end
	else
		return "Off";
	end
end

function MxFH_SaveSettings()
	MxFH.DebugPrint("Saving settings");
	SlaveGirlOptions.Setting[MxFH.Data.Me] = MxFH.Setting;
	getglobal("MxFH_MonitorButton"):SetChecked(MxFH.Setting.Monitor);
	getglobal("MxFH_BroadcastButton"):SetChecked(MxFH.Setting.Broadcast);
	getglobal("MxFH_StartFollowSoundButton"):SetChecked(MxFH.Setting.StartFollowSound);
	getglobal("MxFH_StopFollowSoundButton"):SetChecked(MxFH.Setting.StopFollowSound);
	getglobal("MxFH_LowHealthSoundButton"):SetChecked(MxFH.Setting.LowHealthSound);
	getglobal("MxFH_LowManaSoundButton"):SetChecked(MxFH.Setting.LowManaSound);
	getglobal("MxFH_AutoAcceptGroupInviteButton"):SetChecked(MxFH.Setting.AutoAcceptGroupInvite);
	getglobal("MxFH_AutoAcceptQuestButton"):SetChecked(MxFH.Setting.AutoAcceptQuest);
	getglobal("MxFH_AutoShareQuestButton"):SetChecked(MxFH.Setting.AutoShareQuest);
	getglobal("MxFH_AutoTurnInQuestButton"):SetChecked(MxFH.Setting.AutoTurnInQuest);
	getglobal("MxFH_AutoLeaveGroupButton"):SetChecked(MxFH.Setting.AutoLeaveGroup);
	getglobal("MxFH_AutoRoleButton"):SetChecked(MxFH.Setting.AutoRole);
	if (MxFH.Setting.Master) then
		getglobal("MxFH_MasterInfo"):SetText("Current master: |cFFEEEE00"..MxFH.Setting.Master.."|r");
		getglobal("MxFH_RemoveMasterButton"):Enable();
	else
		getglobal("MxFH_MasterInfo"):SetText("Current master: [|cFFEE0000Nobody|r]");
		getglobal("MxFH_RemoveMasterButton"):Disable();
	end
	getglobal("MxFH_LowHealthSlider"):SetValue(MxFH.Setting.LowHealthThreshold);
	if (MxFH.Setting.LowHealthThreshold > 0) then
		getglobal("MxFH_LowHealthValue"):SetText(MxFH.Setting.LowHealthThreshold.."%");
	else
		getglobal("MxFH_LowHealthValue"):SetText("Disabled");			
	end
	getglobal("MxFH_LowManaSlider"):SetValue(MxFH.Setting.LowManaThreshold);
	if (MxFH.Setting.LowManaThreshold > 0) then
		getglobal("MxFH_LowManaValue"):SetText(MxFH.Setting.LowManaThreshold.."%");
	else
		getglobal("MxFH_LowManaValue"):SetText("Disabled");			
	end
end

function MxFH_ShareQuest(index)
	if (MxFH.Data.ClassicMode) then
		local questLink = GetQuestLogTitle(index);
		SelectQuestLogEntry(index)
		if (GetQuestLogPushable()) then
			QuestLogPushQuest(index)
			MxFH_SendComm("O/S/"..questLink, true);
		elseif (questLink) then
			MxFH_ErrorPrint("Unable to share "..questLink);
			MxFH_SendComm("O/S/"..questLink, true);
		end
	else
		local questLink = GetQuestLink(index) or C_QuestLog.GetTitleForQuestID(index);
		if (C_QuestLog.IsPushableQuest(index)) then
			C_QuestLog.SetSelectedQuest(index);
			QuestLogPushQuest();
			MxFH_SendComm("O/S/"..questLink, true);
		elseif (questLink) then
			MxFH_ErrorPrint("Unable to share "..questLink);
			MxFH_SendComm("O/S/"..questLink, true);
		end
	end
end

-- Check the group for followers
function MxFH_CheckGroup(activelySearch)
	local raidmembers = GetNumGroupMembers();
	local partymembers = GetNumSubgroupMembers();
	local wasMasterInGroup = MxFH.Data.MasterInGroup;
	if (raidmembers > 0 and not UnitExists("raid1")) then
		raidmembers = 0
	end

	MxFH.Data.FollowerInGroup = nil;
	MxFH.Data.MasterInGroup = nil;
	
	if (activelySearch and (raidmembers > 0 or partymembers > 0)) then
		MxFH_SendComm("G/M", true);
		MxFH.DebugPrint("Actively looking for followers.");
	end
	if (raidmembers > 0) then
		for i = 1, raidmembers, 1 do
			local name = UnitName("raid"..i);
			local fullName = MxFH_FullName("raid"..i, true);
			if (MxFH.Data.Followers[name] or MxFH.Data.Followers[fullName]) then
				MxFH.Data.FollowerInGroup = true;
				MxFH.DebugPrint("Follower is in group: Raid"..i);
			end
			if (MxFH_IsMaster(name)) then
				MxFH.Data.MasterInGroup = true;
				MxFH.DebugPrint("Master is in group: Raid"..i);
			end
		end
	elseif (partymembers > 0) then
		for i = 1, partymembers, 1 do
			local name = UnitName("party"..i);
			local fullName = MxFH_FullName("party"..i, true);
			if (MxFH.Data.Followers[name] or MxFH.Data.Followers[fullName]) then
				MxFH.Data.FollowerInGroup = true;
				MxFH.DebugPrint("Follower is in group: Party"..i);
			end
			if (MxFH_IsMaster(name)) then
				MxFH.Data.MasterInGroup = true;
				MxFH.DebugPrint("Master is in group: Party"..i);
			end
		end
	end
	
	if (wasMasterInGroup and MxFH.Setting.AutoLeaveGroup and not MxFH.Data.MasterInGroup and GetTime() > (MxFH.Data.AutoLeaveGroupIgnore or 0)) then
		-- Master is no longer in group, leaving...
		MxFH.DebugPrint("Master is no longer in group, leaving group");
		LeaveParty();
	end

end

function MxFH_ReportToMaster(master)
	if (master and MxFH_IsMaster(master)) then
		C_ChatInfo.SendAddonMessage(MxFH.ModCode, "G/S", "WHISPER", MxFH.Setting.Master);
		MxFH.DebugPrint("Reporting to: "..master);
	end
end

-- Create Addon Menu options and interface
function MxFH_RenderOptions()
	local ConfigurationPanel = CreateFrame("FRAME","MxFH_MainFrame");
	ConfigurationPanel.name = "Multibox Helper";
	InterfaceOptions_AddCategory(ConfigurationPanel);

	local IntroMessageHeader = ConfigurationPanel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	IntroMessageHeader:SetPoint("TOPLEFT", 10, -10);
	IntroMessageHeader:SetText("Multibox Follower Helper "..MxFH.Version);

	local IntroMessageMaster = ConfigurationPanel:CreateFontString("MxFH_MasterInfo", "ARTWORK", "GameFontNormal");
	IntroMessageMaster:SetPoint("TOPLEFT", 10, -35);
	IntroMessageMaster:SetText("Current Master: ");

	local IntroMessageFooter = ConfigurationPanel:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	IntroMessageFooter:SetPoint("TOPLEFT", 10, -75);
	IntroMessageFooter:SetText("Click a player and click |cFFFFFFFFSet|r to set them as your master.");

	local SetMasterButton = CreateFrame("Button", "MxFH_SetMasterButton", ConfigurationPanel, "OptionsButtonTemplate");
	SetMasterButton:SetPoint("TOPLEFT", 220, -30);
	SetMasterButton.tooltip = "Set this person as your master";
	SetMasterButton:SetScript("OnClick",MxFH_Option_SetMaster);
	getglobal(SetMasterButton:GetName().."Text"):SetText("Set");

	local RemoveMasterButton = CreateFrame("Button", "MxFH_RemoveMasterButton", ConfigurationPanel, "OptionsButtonTemplate");
	RemoveMasterButton:SetPoint("TOPLEFT", 310, -30);
	RemoveMasterButton.tooltip = "Remove this person as your master";
	RemoveMasterButton:SetScript("OnClick",MxFH_Option_NoMaster);
	getglobal(RemoveMasterButton:GetName().."Text"):SetText("Remove");

	
	-- Communications

	local CommunicationPanel = CreateFrame("FRAME","MxFH_CommunicationFrame");
	CommunicationPanel.name = "Communications";
	CommunicationPanel.parent = ConfigurationPanel.name;
	InterfaceOptions_AddCategory(CommunicationPanel);

	local MonitorButton = CreateFrame("CheckButton", "MxFH_MonitorButton", CommunicationPanel, "ChatConfigCheckButtonTemplate");
	MonitorButton:SetPoint("TOPLEFT", 10, -15)
	MonitorButton.tooltip = "";
	getglobal(MonitorButton:GetName().."Text"):SetText(" Display follower communications to chat window");

	local BroadcastButton = CreateFrame("CheckButton", "MxFH_BroadcastButton", CommunicationPanel, "ChatConfigCheckButtonTemplate");
	BroadcastButton:SetPoint("TOPLEFT", 10, -45)
	BroadcastButton.tooltip = "";
	getglobal(BroadcastButton:GetName().."Text"):SetText(" Broadcast error messages to master (when grouped)");

	-- Groups/Quests

	local QuestPanel = CreateFrame("FRAME","MxFH_QuestFrame");
	QuestPanel.name = "Groups/Quests";
	QuestPanel.parent = ConfigurationPanel.name;
	InterfaceOptions_AddCategory(QuestPanel);

	local AutoAcceptGroupInviteButton = CreateFrame("CheckButton", "MxFH_AutoAcceptGroupInviteButton", QuestPanel, "ChatConfigCheckButtonTemplate");
	AutoAcceptGroupInviteButton:SetPoint("TOPLEFT", 10, -15)
	AutoAcceptGroupInviteButton.tooltip = "";
	getglobal(AutoAcceptGroupInviteButton:GetName().."Text"):SetText(" Auto-accept master's group invite");

	local AutoAcceptQuestButton = CreateFrame("CheckButton", "MxFH_AutoAcceptQuestButton", QuestPanel, "ChatConfigCheckButtonTemplate");
	AutoAcceptQuestButton:SetPoint("TOPLEFT", 10, -45)
	AutoAcceptQuestButton.tooltip = "";
	getglobal(AutoAcceptQuestButton:GetName().."Text"):SetText(" Auto-accept shared quests");

	local AutoShareQuestButton = CreateFrame("CheckButton", "MxFH_AutoShareQuestButton", QuestPanel, "ChatConfigCheckButtonTemplate");
	AutoShareQuestButton:SetPoint("TOPLEFT", 10, -75)
	AutoShareQuestButton.tooltip = "";
	getglobal(AutoShareQuestButton:GetName().."Text"):SetText(" Auto-share quests (when grouped with a master or follower)");

	local AutoTurnInQuestButton = CreateFrame("CheckButton", "MxFH_AutoTurnInQuestButton", QuestPanel, "ChatConfigCheckButtonTemplate");
	AutoTurnInQuestButton:SetPoint("TOPLEFT", 10, -105)
	AutoTurnInQuestButton.tooltip = "";
	getglobal(AutoTurnInQuestButton:GetName().."Text"):SetText(" Auto-start and complete quests (that don't have rewards)");

	local AutoLeaveGroupButton = CreateFrame("CheckButton", "MxFH_AutoLeaveGroupButton", QuestPanel, "ChatConfigCheckButtonTemplate");
	AutoLeaveGroupButton:SetPoint("TOPLEFT", 10, -135)
	AutoLeaveGroupButton.tooltip = "";
	getglobal(AutoLeaveGroupButton:GetName().."Text"):SetText(" Auto-leave groups when master leaves the group");

	local AutoRoleButton = CreateFrame("CheckButton", "MxFH_AutoRoleButton", QuestPanel, "ChatConfigCheckButtonTemplate");
	AutoRoleButton:SetPoint("TOPLEFT", 10, -165)
	AutoRoleButton.tooltip = "";
	getglobal(AutoRoleButton:GetName().."Text"):SetText(" Auto-choose roles based on current specialization and hide selection popups");

	-- Sounds

	local SoundPanel = CreateFrame("FRAME","MxFH_SoundFrame");
	SoundPanel.name = "Sounds";
	SoundPanel.parent = ConfigurationPanel.name;
	InterfaceOptions_AddCategory(SoundPanel);

	local StartFollowSoundButton = CreateFrame("CheckButton", "MxFH_StartFollowSoundButton", SoundPanel, "ChatConfigCheckButtonTemplate");
	StartFollowSoundButton:SetPoint("TOPLEFT", 10, -15)
	StartFollowSoundButton.tooltip = "";
	getglobal(StartFollowSoundButton:GetName().."Text"):SetText(" Start Following");

	local StartFollowSoundTestButton = CreateFrame("Button", "MxFH_StartFollowSoundTestButton", SoundPanel, "OptionsButtonTemplate");
	StartFollowSoundTestButton:SetPoint("TOPLEFT", 300, -15);
	StartFollowSoundTestButton.tooltip = "Play the sound of when your character is following another";
	StartFollowSoundTestButton:SetScript("OnClick",MxFH_Option_StartFollowTest);
	getglobal(StartFollowSoundTestButton:GetName().."Text"):SetText("Test");

	local StopFollowSoundButton = CreateFrame("CheckButton", "MxFH_StopFollowSoundButton", SoundPanel, "ChatConfigCheckButtonTemplate");
	StopFollowSoundButton:SetPoint("TOPLEFT", 10, -45)
	StopFollowSoundButton.tooltip = "";
	getglobal(StopFollowSoundButton:GetName().."Text"):SetText(" Stop Following");

	local StopFollowSoundTestButton = CreateFrame("Button", "MxFH_StopFollowSoundTestButton", SoundPanel, "OptionsButtonTemplate");
	StopFollowSoundTestButton:SetPoint("TOPLEFT", 300, -45);
	StopFollowSoundTestButton.tooltip = "Play the sound of when your character stops following another";
	StopFollowSoundTestButton:SetScript("OnClick",MxFH_Option_StopFollowTest);
	getglobal(StopFollowSoundTestButton:GetName().."Text"):SetText("Test");

	local LowHealthSoundButton = CreateFrame("CheckButton", "MxFH_LowHealthSoundButton", SoundPanel, "ChatConfigCheckButtonTemplate");
	LowHealthSoundButton:SetPoint("TOPLEFT", 10, -75)
	LowHealthSoundButton.tooltip = "";
	getglobal(LowHealthSoundButton:GetName().."Text"):SetText(" Low Health");

	local LowHealthSoundTestButton = CreateFrame("Button", "MxFH_LowHealthSoundTestButton", SoundPanel, "OptionsButtonTemplate");
	LowHealthSoundTestButton:SetPoint("TOPLEFT", 300, -75);
	LowHealthSoundTestButton.tooltip = "Play the sound of when your character is following another";
	LowHealthSoundTestButton:SetScript("OnClick",MxFH_Option_LowHealthTest);
	getglobal(LowHealthSoundTestButton:GetName().."Text"):SetText("Test");

	local LowHealthSlider = CreateFrame("Slider", "MxFH_LowHealthSlider", SoundPanel, "OptionsSliderTemplate");
	LowHealthSlider:SetPoint("TOPLEFT", 45, -115);
	LowHealthSlider.tooltip = "Percentage of HP to sound alert";
	LowHealthSlider:SetScript("OnValueChanged",MxFH_Option_SetLowHealth);
	getglobal(MxFH_LowHealthSlider:GetName().."Text"):SetText("Health %");
	getglobal(MxFH_LowHealthSlider:GetName().."High"):SetText("Max");
	getglobal(MxFH_LowHealthSlider:GetName().."Low"):SetText("Min");
	LowHealthSlider:SetMinMaxValues(0,100);
	LowHealthSlider:SetValueStep(1);
	LowHealthSlider:SetValue(MxFH.Setting.LowHealthThreshold);

	local LowHealthValue = SoundPanel:CreateFontString("MxFH_LowHealthValue", "ARTWORK", "GameFontNormal");
	LowHealthValue:SetPoint("TOPLEFT", 200, -115);
	LowHealthValue:SetText(MxFH.Setting.LowHealthThreshold.."%");

	local LowManaSoundButton = CreateFrame("CheckButton", "MxFH_LowManaSoundButton", SoundPanel, "ChatConfigCheckButtonTemplate");
	LowManaSoundButton:SetPoint("TOPLEFT", 10, -145)
	LowManaSoundButton.tooltip = "";
	getglobal(LowManaSoundButton:GetName().."Text"):SetText(" Low Mana");

	local LowManaSoundTestButton = CreateFrame("Button", "MxFH_LowManaSoundTestButton", SoundPanel, "OptionsButtonTemplate");
	LowManaSoundTestButton:SetPoint("TOPLEFT", 300, -145);
	LowManaSoundTestButton.tooltip = "Play the sound of when your character is following another";
	LowManaSoundTestButton:SetScript("OnClick",MxFH_Option_LowManaTest);
	getglobal(LowManaSoundTestButton:GetName().."Text"):SetText("Test");

	local LowManaSlider = CreateFrame("Slider", "MxFH_LowManaSlider", SoundPanel, "OptionsSliderTemplate");
	LowManaSlider:SetPoint("TOPLEFT", 45, -185);
	LowManaSlider.tooltip = "Percentage of Mana to sound alert";
	LowManaSlider:SetScript("OnValueChanged",MxFH_Option_SetLowMana);
	getglobal(MxFH_LowManaSlider:GetName().."Text"):SetText("Mana %");
	getglobal(MxFH_LowManaSlider:GetName().."High"):SetText("Max");
	getglobal(MxFH_LowManaSlider:GetName().."Low"):SetText("Min");
	LowManaSlider:SetMinMaxValues(0,100);
	LowManaSlider:SetValueStep(1);
	LowManaSlider:SetValue(MxFH.Data.LowManaThreshold);

	local LowManaValue = SoundPanel:CreateFontString("MxFH_LowManaValue", "ARTWORK", "GameFontNormal");
	LowManaValue:SetPoint("TOPLEFT", 200, -185);
	LowManaValue:SetText(MxFH.Data.LowManaThreshold.."%");
	
	ConfigurationPanel.okay = 
		function (self)
			MxFH.Setting.Monitor = MonitorButton:GetChecked();
			MxFH.Setting.Broadcast = BroadcastButton:GetChecked();
			MxFH.Setting.StartFollowSound = StartFollowSoundButton:GetChecked();
			MxFH.Setting.StopFollowSound = StopFollowSoundButton:GetChecked();
			MxFH.Setting.LowHealthSound = LowHealthSoundButton:GetChecked();
			MxFH.Setting.LowManaSound = LowManaSoundButton:GetChecked();
			MxFH.Setting.AutoAcceptGroupInvite = AutoAcceptGroupInviteButton:GetChecked();
			MxFH.Setting.AutoAcceptQuest = AutoAcceptQuestButton:GetChecked();
			MxFH.Setting.AutoShareQuest = AutoShareQuestButton:GetChecked();
			MxFH.Setting.AutoTurnInQuest = AutoTurnInQuestButton:GetChecked();
			MxFH.Setting.AutoLeaveGroup = AutoLeaveGroupButton:GetChecked();
			MxFH.Setting.AutoRole = AutoRoleButton:GetChecked();
			MxFH.Setting.LowHealthThreshold = getglobal("MxFH_LowHealthSlider"):GetValue();
			MxFH.Setting.LowManaThreshold = getglobal("MxFH_LowManaSlider"):GetValue();
			MxFH_SaveSettings();
		end
	ConfigurationPanel.cancel = 
		function (self)

		end
	ConfigurationPanel.default = 
		function (self)
			MxFH_ResetDefaults();
			MxFH_SaveSettings();
		end
	SoundPanel.default = 
		function (self)
			MxFH_ResetDefaults();
			MxFH_SaveSettings();
		end
	QuestPanel.default = 
		function (self)
			MxFH_ResetDefaults();
			MxFH_SaveSettings();
		end
	CommunicationPanel.default = 
		function (self)
			MxFH_ResetDefaults();
			MxFH_SaveSettings();
		end
end

function MxFH_Option_StartFollowTest()
	MxFH_PlaySound("StartFollow");
end

function MxFH_Option_StopFollowTest()
	MxFH_PlaySound("StopFollow");
end

function MxFH_Option_LowHealthTest()
	MxFH_PlaySound("LowHealth");
end

function MxFH_Option_LowManaTest()
	MxFH_PlaySound("LowMana");
end

function MxFH_Option_SetMaster()
	if (UnitName("target")) then
		MxFH_Command_Master(nil);
	end
end

function MxFH_Option_NoMaster()
	MxFH_Command_Master("c");
end

function MxFH_Option_SetLowHealth()
	MxFH.Data.LowHealthThreshold = getglobal("MxFH_LowHealthSlider"):GetValue();
	if (getglobal("MxFH_LowHealthValue")) then
		if (MxFH.Data.LowHealthThreshold > 0) then
			getglobal("MxFH_LowHealthValue"):SetText(MxFH.Data.LowHealthThreshold.."%");
		else
			getglobal("MxFH_LowHealthValue"):SetText("Disabled");			
		end
	end
end

function MxFH_Option_SetLowMana()
	MxFH.Data.LowManaThreshold = getglobal("MxFH_LowManaSlider"):GetValue();
	if (getglobal("MxFH_LowManaValue")) then
		if (MxFH.Data.LowManaThreshold > 0) then
			getglobal("MxFH_LowManaValue"):SetText(MxFH.Data.LowManaThreshold.."%");
		else
			getglobal("MxFH_LowManaValue"):SetText("Disabled");			
		end
	end
end

function MxFH_IsMaster(name)
	if (MxFH.Setting.Master and name) then
		if (MxFH.Setting.Master == name) then
			return true;
		end
		if (MxFH.Setting.Master.."-"..MxFH.Data.Realm == name) then
			return true;
		end
		if (MxFH.Setting.Master == name.."-"..MxFH.Data.Realm) then
			return true;
		end
	end
end

function MxFH_FullName(sUnit, bForceRealm)
	local name, realm = UnitName(sUnit);
	if (realm and realm ~= "") then
		return name.."-"..realm;
	elseif (name and bForceRealm) then
		return name.."-"..MxFH.Data.Realm;
	else
		return name;
	end
end

function MxFH.StopFollowing(justStarted)
	if (justStarted) then
		MxFH.DebugPrint("Ignore stop following");
		return;
	end
	
	if (MxFH.Setting.StopFollowSound) then
		MxFH_PlaySound("StopFollow");
	end
	MxFH_SendComm("F/X");
end

function MxFH.StartFollowing(follower, justStopped)
	if (follower) then
		if (MxFH.Data.LastFollowingTarget and MxFH.Data.LastFollowingTarget == follower and justStopped) then
			MxFH.DebugPrint("Ignore start following");
			return;
		end

		MxFH_SendComm("F/S/"..follower);
		if (MxFH.Data.LastFollowingTarget ~= follower) then
			MxFH.Data.LastFollowingTarget = follower;
		end
	else
		MxFH_SendComm("F/S");
	end
	if (MxFH.Setting.StartFollowSound) then
		MxFH_PlaySound("StartFollow");
	end
end

function MxFH.StripRealm(sText)
	local result = sText;
	local i = string.find(result, "-");
	if (i) then
		result = string.sub(result, 1, i-1);
	end
	return result;
end

function MxFH_Command_Debug()
		if (MxFH.Data.DebugMode) then
		MxFH.ChatPrint("Debug mode is off.")
		MxFH.Data.DebugMode = nil;
	else
		MxFH.ChatPrint("Debug mode is on.")
		MxFH.Data.DebugMode = true;
	end
end
